from typing import Any

from hatchet_sdk.runnables.workflow import Workflow
from hatchet_sdk.runnables.types import WorkflowConfig
from hatchet_sdk.clients.listeners.run_event_listener import StepRunEventType
from pydantic import (
    BaseModel,
    HttpUrl,
    ConfigDict,
    field_serializer,
)
from pydantic.json_schema import SkipJsonSchema
from fastapi import FastAPI
from typer import Typer


class Plugin(BaseModel):
    id: str
    module: str
    description: str


class APIPlugin(Plugin):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    api: SkipJsonSchema[FastAPI]

    @field_serializer("api")
    def serialize_api(self, api: FastAPI, _info):
        pass


class CLIPlugin(Plugin):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    cli: SkipJsonSchema[Typer]

    @field_serializer("cli")
    def serialize_cli(self, cli: Typer, _info):
        pass


def serialize_workflow_config(config: WorkflowConfig) -> dict[str, Any]:
    return config.model_json_schema()


class WorkflowMixin(BaseModel):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    wf: SkipJsonSchema[Workflow]

    @field_serializer("wf")
    def serialize_workflow(self, wf: Workflow, _info):
        return {
            "config": wf.config.model_dump(
                mode="json",
                exclude_defaults=True,
                fallback=serialize_workflow_config,
            ),
            "id": wf.id,
            "name": wf.name,
            "service_name": wf.service_name,
            "tasks": [task.name for task in wf.tasks],
        }


class WorkflowPlugin(WorkflowMixin, Plugin):
    pass


class WorkflowTriggerResult(BaseModel):
    wfRunId: str


class WorkflowRunResult(WorkflowMixin):
    wfRunId: str
    result: dict[str, Any]
    payload: str | None = None


class SettingsPlugin(Plugin): ...


class Plugins(BaseModel):
    api: dict[str, APIPlugin] = {}
    cli: dict[str, CLIPlugin] = {}
    workflow: dict[str, WorkflowPlugin] = {}
    settings: dict[str, SettingsPlugin] = {}


class ServerInfo(BaseModel):
    version: str
    hatchet_url: HttpUrl
    api_doc: str
    plugins: Plugins = Plugins()


class HatchetStepEvent(BaseModel):
    wfId: str
    wfRunId: str
    type: StepRunEventType
    payload: str | None = None
    result: dict[str, Any]
